/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.client.gui;

import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractListModel;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import kgrid.client.workflow.Consts;
import kgrid.client.workflow.DatasetNode;
import kgrid.client.workflow.Edge;
import kgrid.client.workflow.ModelNode;
import kgrid.client.workflow.ToolNode;
import kgrid.client.workflow.Workflow;
import kgrid.client.workflow.WorkflowNode;

import org.jdom.Element;

/**
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class PropertiesPanel extends JPanel implements TableModelListener,
		ActionListener, Consts {

	private static final long serialVersionUID = 1L;
	public final static String EMPTY_PANEL = "Empty panel";
	public final static String NODE_PANEL = "Node properties panel";
	public final static String EDGE_PANEL = "Edge properties panel";
	public final static String MODEL_PANEL = "Model properties panel";
	final static String MODEL_CHANGED = "Model location changed";

	final static String KUSTRING = "KDS URL";
	final static String KUCHANGED = "KDS changed";
	final static String SPSTRING = "Search parameters";

	private Workflow workflow;

	private WorkflowNode node;
	// private String[] columnNames = { "Property", "Value" };

	private Color tableBkgColor;

	protected JPanel nodePanel = new JPanel();
	protected JPanel kdsUrlPanel = new JPanel(new FlowLayout());
	private ButtonGroup urlOrSearchGroup = new ButtonGroup();
	private JRadioButton kdsUrlRadio;
	private JTextField kdsUrlField = new JTextField(23);

	private JRadioButton srcRadio;
	private JScrollPane scrollSrcPrmTablePane;
	protected JTable srcPrmTable = new JTable(1, 2);
	protected DefaultTableModel srcPrmTableModel = (DefaultTableModel) srcPrmTable
			.getModel();

	protected JPanel argPanel = new JPanel();
	protected JScrollPane scrollArgTablePane;
	protected JTable argTable = new JTable(1, 2);
	protected DefaultTableModel argTableModel = (DefaultTableModel) argTable
			.getModel();

	protected JPanel edgePanel = new JPanel();
	protected JPanel emptyPanel = new JPanel();

	protected JPanel modelPanel = new JPanel();
	private JTextField locationField = new JTextField(23);

	private Edge edge;
	private ButtonGroup edgeRadioGroup = new ButtonGroup();
	private JRadioButton datasetRadio;
	private JRadioButton trainRadio;
	private JRadioButton testRadio;

	private CardLayout layout;

	public PropertiesPanel(Workflow w) {
		// super(new GridLayout(1, 1));
		super(new CardLayout());
		layout = (CardLayout) getLayout();
		workflow = w;
		add(emptyPanel, EMPTY_PANEL);
		addNodePanel();
		addModelPanel();
		addEdgePanel();
		layout.show(this, EMPTY_PANEL);
	}

	private void addNodePanel() {
		nodePanel.setLayout(new BoxLayout(nodePanel, BoxLayout.PAGE_AXIS));
		addNodeKdsUrlPanel();
		addNodeSearchPanel();
		addArgumentsPanel();
		nodePanel.add(Box.createVerticalGlue());
		add(nodePanel, NODE_PANEL);
	}

	private void addModelPanel() {
		locationField.setActionCommand(MODEL_CHANGED);
		locationField.addActionListener(this);
		Dimension d = locationField.getMaximumSize();
		d.setSize(d.getWidth(), 20);
		locationField.setMaximumSize(d);

		modelPanel.setLayout(new BoxLayout(modelPanel, BoxLayout.PAGE_AXIS));
		modelPanel.add(new JLabel(LOCATION));
		modelPanel.add(locationField);
		modelPanel.add(Box.createVerticalGlue());
		add(modelPanel, MODEL_PANEL);
	}

	private void addNodeKdsUrlPanel() {
		kdsUrlRadio = new JRadioButton(KUSTRING);
		kdsUrlRadio.setActionCommand(KUSTRING);
		kdsUrlRadio.setSelected(true);
		kdsUrlRadio.addActionListener(this);
		kdsUrlRadio.setAlignmentX(LEFT_ALIGNMENT);

		kdsUrlField.setText(KDSURL_PREFIX);
		kdsUrlField.setActionCommand(KUCHANGED);
		kdsUrlField.addActionListener(this);

		urlOrSearchGroup.add(kdsUrlRadio);

		nodePanel.add(kdsUrlRadio);
		nodePanel.add(kdsUrlField);
	}

	private void addNodeSearchPanel() {
		initSrcPrmTable();
		srcRadio = new JRadioButton(SPSTRING);
		srcRadio.setActionCommand(SPSTRING);
		srcRadio.setAlignmentX(LEFT_ALIGNMENT);

		urlOrSearchGroup.add(srcRadio);
		srcRadio.addActionListener(this);
		scrollSrcPrmTablePane = new JScrollPane(srcPrmTable);
		scrollSrcPrmTablePane.setBorder(new EmptyBorder(0, 0, 0, 0));

		Dimension d = scrollSrcPrmTablePane.getMaximumSize();
		d.setSize(d.getWidth(), 20);
		scrollSrcPrmTablePane.setMaximumSize(d);

		nodePanel.add(srcRadio);
		nodePanel.add(scrollSrcPrmTablePane);
	}

	private void addArgumentsPanel() {
		initArgTable();
		argPanel.setLayout(new BoxLayout(argPanel, BoxLayout.PAGE_AXIS));
		argPanel.setAlignmentX(LEFT_ALIGNMENT);
		scrollArgTablePane = new JScrollPane(argTable);
		scrollArgTablePane.setBorder(new EmptyBorder(0, 0, 0, 0));
		scrollArgTablePane.setAlignmentX(LEFT_ALIGNMENT);

		JPanel argLabelButtonPanel = new JPanel();
		argLabelButtonPanel.add(new JLabel("Arguments"));
		JButton addButton = new JButton("+");
		addButton.addActionListener(this);
		argLabelButtonPanel.add(addButton, "+");
		argLabelButtonPanel.setAlignmentX(LEFT_ALIGNMENT);

		argPanel.add(argLabelButtonPanel);
		argPanel.add(scrollArgTablePane);
		argPanel.setVisible(false);
		nodePanel.add(argPanel);
	}

	private void initSrcPrmTable() {
		srcPrmTable.setTableHeader(null);
		/*
		 * table.getTableHeader().getColumnModel().getColumn(0).setHeaderRenderer
		 * ( new DefaultTableCellHeaderRenderer() { private static final long
		 * serialVersionUID = 1L;
		 * 
		 * @Override public void setHorizontalAlignment(int alignment) {
		 * super.setHorizontalAlignment(JLabel.CENTER); } });
		 * table.getTableHeader
		 * ().getColumnModel().getColumn(1).setHeaderRenderer( new
		 * DefaultTableCellHeaderRenderer() { private static final long
		 * serialVersionUID = 1L;
		 * 
		 * @Override public void setHorizontalAlignment(int alignment) {
		 * super.setHorizontalAlignment(JLabel.CENTER); } });
		 */
		srcPrmTable.setIntercellSpacing(new Dimension(4, 4));
		tableBkgColor = srcPrmTable.getBackground();

		// table.getColumnModel().getColumn(0).setHeaderValue(columnNames[0]);
		srcPrmTable.getColumnModel().getColumn(0)
				.setCellRenderer(new DefaultTableCellRenderer() {
					private static final long serialVersionUID = 1L;

					@Override
					public void setHorizontalAlignment(int alignment) {
						super.setHorizontalAlignment(JLabel.RIGHT);
					}
				});
		srcPrmTable.getColumnModel().getColumn(0).setMinWidth(90);
		srcPrmTable.getColumnModel().getColumn(0)
				.setCellEditor(new EmptyTableCellEditor());

		// table.getColumnModel().getColumn(1).setHeaderValue(columnNames[1]);

		srcPrmTable.setCellSelectionEnabled(false);
		srcPrmTable.getModel().addTableModelListener(this);
		// table.getTableHeader().setReorderingAllowed(false);
		srcPrmTable.setRowHeight(srcPrmTable.getRowHeight() + 2);
		srcPrmTable.addMouseListener(new TableMouseListener(workflow, this));
	}

	private void initArgTable() {
		argTable.setBackground(Color.WHITE);
		argTable.setTableHeader(null);
		argTable.setIntercellSpacing(new Dimension(4, 4));

		argTable.getColumnModel().getColumn(0)
				.setCellRenderer(new DefaultTableCellRenderer() {
					private static final long serialVersionUID = 1L;

					@Override
					public void setHorizontalAlignment(int alignment) {
						super.setHorizontalAlignment(JLabel.RIGHT);
					}
				});
		argTable.getColumnModel().getColumn(0).setMinWidth(90);
		argTable.getColumnModel().getColumn(0)
				.setCellEditor(new EmptyTableCellEditor());

		argTable.setCellSelectionEnabled(false);
		argTable.getModel().addTableModelListener(this);
		argTable.setRowHeight(argTable.getRowHeight() + 2);
		argTable.addMouseListener(new TableMouseListener(workflow, this));
	}

	private void addEdgePanel() {

		datasetRadio = new JRadioButton(Edge.DATASET);
		datasetRadio.setActionCommand(Edge.DATASET);

		trainRadio = new JRadioButton(Edge.TRAIN);
		trainRadio.setActionCommand(Edge.TRAIN);

		testRadio = new JRadioButton(Edge.TEST);
		testRadio.setActionCommand(Edge.TEST);

		edgeRadioGroup.add(datasetRadio);
		edgeRadioGroup.add(trainRadio);
		edgeRadioGroup.add(testRadio);

		edgePanel.add(datasetRadio);
		edgePanel.add(trainRadio);
		edgePanel.add(testRadio);

		datasetRadio.addActionListener(this);
		trainRadio.addActionListener(this);
		testRadio.addActionListener(this);

		add(edgePanel, EDGE_PANEL);
	}

	public void changeTable(WorkflowNode node) {
		srcPrmTableModel.getDataVector().removeAllElements();
		this.node = node;
		LinkedHashMap<String, Object> properties = node.getProperties();

		/*
		 * TODO change 1st column width in function of the node type if (node
		 * instanceof DatasetNode) {
		 * table.getColumnModel().getColumn(0).setWidth(90); } else if (node
		 * instanceof ToolNode) {
		 * table.getColumnModel().getColumn(0).setWidth(70); } else {
		 * table.getColumnModel().getColumn(0).setWidth(50); }
		 */
		kdsUrlField.setText(KDSURL_PREFIX);
		setSearchParamSelected();
		for (Map.Entry<String, Object> e : properties.entrySet()) {
			if (e.getKey().equals(LOCATION)) {
				locationField.setText((String) e.getValue());
				locationField.setCaretPosition(0);
			} else if (e.getKey().equals(PKDS)) {
				String kdsUrl = (String) e.getValue();
				if (!kdsUrl.isEmpty() && !kdsUrl.equals(KDSURL_PREFIX)) {
					kdsUrlField.setText((String) e.getValue());
					kdsUrlField.setCaretPosition(0);
					setKdsUrlSelected();
				}
			} else if (e.getKey().equals(PATTR)) {
				LinkedList<LinkedHashMap<String, Object>> attributes = (LinkedList<LinkedHashMap<String, Object>>) e
						.getValue();
				for (LinkedHashMap<String, Object> attr : attributes) {
					for (Map.Entry<String, Object> attrPr : attr.entrySet()) {
						if (attrPr.getKey().equals(PARAM)) {
							LinkedList<String> parameters = (LinkedList<String>) attrPr
									.getValue();
							for (String p : parameters) {
								srcPrmTableModel.addRow(new Object[] {
										"Parameter", p });
							}
						} else {
							srcPrmTableModel.addRow(new Object[] {
									attrPr.getKey(), attrPr.getValue() });
						}
					}
				}
			} else if (e.getKey().equals(PARGS)) {
				LinkedHashMap<String, String> args = (LinkedHashMap<String, String>) e
						.getValue();
				for (Map.Entry<String, String> arg : args.entrySet()) {
					argTableModel.addRow(new Object[] { arg.getKey(),
							arg.getValue() });
				}
			} else if (e.getKey().equals(PRES)) {
				LinkedList<ModelNode.Result> results = (LinkedList<ModelNode.Result>) e
						.getValue();
				for (ModelNode.Result res : results) {
					srcPrmTableModel.addRow(new Object[] { res.getDate(),
							res.getFileName() });
					int lastRow = srcPrmTableModel.getRowCount() - 1;
					// TODO make the results not editable
				}
			} else {
				srcPrmTableModel
						.addRow(new Object[] { e.getKey(), e.getValue() });
			}
		}
		if (kdsUrlRadio.isSelected()) {
			setKdsUrlSelected();
		} else {
			setSearchParamSelected();
		}
		if (node instanceof ModelNode) {
			layout.show(this, MODEL_PANEL);
		} else {
			if (node instanceof DatasetNode) {
				argPanel.setVisible(false);
			} else {
				argPanel.setVisible(true);
			}
			layout.show(this, NODE_PANEL);
		}
	}

	private void cleanSrcTable(WorkflowNode node) {
		this.node = node;
		node.clearSearchParameters();
		srcPrmTableModel.getDataVector().removeAllElements();
		if (node instanceof ToolNode) {
			for (Map.Entry<String, Object> e : node.getProperties().entrySet()) {
				if (!e.getKey().equals(PARGS) && !e.getKey().equals(PKDS)) {
					srcPrmTableModel.addRow(new Object[] { e.getKey(),
							e.getValue() });
				}
			}
		} else {
			for (Map.Entry<String, Object> e : node.getProperties().entrySet()) {
				if (e.getKey().equals(PATTR)) {
					LinkedList<LinkedHashMap<String, Object>> attributes = (LinkedList<LinkedHashMap<String, Object>>) e
							.getValue();
					for (LinkedHashMap<String, Object> attr : attributes) {
						for (Map.Entry<String, Object> attrPr : attr.entrySet()) {
							if (attrPr.getKey().equals(PARAM)) {
								LinkedList<String> parameters = (LinkedList<String>) attrPr
										.getValue();
								for (String p : parameters) {
									srcPrmTableModel.addRow(new Object[] {
											"Parameter", p });
								}
							} else {
								srcPrmTableModel.addRow(new Object[] {
										attrPr.getKey(), attrPr.getValue() });
							}
						}
					}
				} else if (!e.getKey().equals(PKDS)) {
					srcPrmTableModel.addRow(new Object[] { e.getKey(),
							e.getValue() });
				}
			}
		}

	}

	public void updateArgTable(WorkflowNode node) {
		clearTable(argTable);
		this.node = node;
		LinkedHashMap<String, Object> properties = node.getProperties();
		for (Map.Entry<String, Object> e : properties.entrySet()) {
			if (e.getKey().equals(PARGS)) {
				LinkedHashMap<String, String> args = (LinkedHashMap<String, String>) e
						.getValue();
				for (Map.Entry<String, String> arg : args.entrySet()) {
					argTableModel.addRow(new Object[] { arg.getKey(),
							arg.getValue() });
				}
			}
		}
	}

	private void setKdsUrlSelected() {
		kdsUrlRadio.setSelected(true);
		kdsUrlField.setEnabled(true);
		srcPrmTable.setBackground(tableBkgColor);
		srcPrmTable.setEnabled(false);
	}

	private void setSearchParamSelected() {
		srcRadio.setSelected(true);
		kdsUrlField.setEnabled(false);
		srcPrmTable.setBackground(Color.WHITE);
		srcPrmTable.setEnabled(true);
	}

	private void clearTable(JTable table) {
		DefaultTableModel model = (DefaultTableModel) table.getModel();
		model.getDataVector().removeAllElements();
		table.setSize(0, 0);
		while (model.getRowCount() > 0) {
			model.removeRow(0);
		}
	}

	public void clear() {
		clearTable(srcPrmTable);
		clearTable(argTable);
		layout.show(this, EMPTY_PANEL);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (command.equals(KUSTRING)) {
			setKdsUrlSelected();
		} else if (command.equals(SPSTRING)) {
			setSearchParamSelected();
		} else if (command.equals(KUCHANGED)) {
			node.setProperty(0, WorkflowNode.PKDS, kdsUrlField.getText());
			cleanSrcTable(node);
			node.resetStatus();
			workflow.setChanged(true);
		} else if (command.equals(MODEL_CHANGED)) {
			node.setProperty(0, LOCATION, locationField.getText());
			workflow.setChanged(true);
		} else if (command.equals("+")) {
			String argName = JOptionPane.showInputDialog(this, "Argument name");
			if (argName != null && !argName.isEmpty()) {
				try {
					((ToolNode) node).addArgument(argName);
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(this, ex.getMessage());
				}
				this.updateArgTable(node);
			}
		} else {
			edge.setType(command);
			workflow.setChanged(true);
		}
	}

	@Override
	public void tableChanged(TableModelEvent e) {
		int row = e.getFirstRow();
		int column = e.getColumn();
		if (column >= 0) {
			TableModel model = (TableModel) e.getSource();
			// String columnName = model.getColumnName(column);
			String property = (String) model.getValueAt(row, column - 1);
			String value = (String) model.getValueAt(row, column);
			if (argumentEntered(model)) {
				((ToolNode) node).setArgument(row, property, value);
			} else {
				node.setProperty(row, property, value);
			}
			if (brokenKDSURL() || searchParametersSelected()) {
				// the user has not entered a kds url in the kds url field
				node.setProperty(0, WorkflowNode.PKDS, KDSURL_PREFIX);
				kdsUrlField.setText(KDSURL_PREFIX);
				node.resetStatus();
			}
			workflow.setChanged(true);
		}
	}

	private boolean brokenKDSURL() {
		return !(node instanceof ModelNode) && !kdsUrlField.getText().isEmpty()
				&& !kdsUrlField.getText().startsWith(KDSURL_PREFIX);
	}

	private boolean searchParametersSelected() {
		return srcRadio.isSelected();
	}

	private boolean argumentEntered(TableModel model) {
		return node instanceof ToolNode && model.equals(argTable.getModel());
	}

	public void setEdgeType(Edge edge) {
		this.edge = edge;
		layout.show(this, EDGE_PANEL);
		edgeRadioGroup.clearSelection();
		if (edge.getType().equals(Edge.DATASET)) {
			datasetRadio.setSelected(true);
		} else if (edge.getType().equals(Edge.TRAIN)) {
			trainRadio.setSelected(true);
		} else if (edge.getType().equals(Edge.TEST)) {
			testRadio.setSelected(true);
		}
	}

	public JTable getSrcPrmTable() {
		return srcPrmTable;
	}

	public JTable getArgTable() {
		return argTable;
	}

	public String getType() {
		for (Component c : getComponents()) {
			if (c.isVisible()) {
				if (c.equals(edgePanel)) {
					return EDGE_PANEL;
				}
				if (c.equals(nodePanel)) {
					return NODE_PANEL;
				}
				if (c.equals(modelPanel)) {
					return MODEL_PANEL;
				}
				if (c.equals(emptyPanel)) {
					return EMPTY_PANEL;
				}
			}
		}
		return "";
	}

	private LinkedHashMap<String, String> getHashMapContent() {
		LinkedHashMap<String, String> map = new LinkedHashMap<String, String>();
		for (Component c : getComponents()) {
			if (c.isVisible()) {
				if (c.equals(nodePanel)) {
					int totalRows = srcPrmTableModel.getRowCount();
					for (int i = 0; i < totalRows; i++) {
						String value = (String) srcPrmTableModel.getValueAt(i,
								1);
						if (!value.isEmpty()) {
							String key = (String) srcPrmTableModel.getValueAt(
									i, 0);
							map.put(key, value);
						}
					}
					return map;
				}
			}
		}
		return map;
	}

	/**
	 * Returns an array of strings containing "property=value" pairs.
	 */
	public String[] getStringContent() {
		LinkedHashMap<String, String> map = getHashMapContent();
		int size = map.size();
		int formatNumber = 0;
		String formatString = "";
		if (map.containsKey(kgrid.client.workflow.Consts.PASEP)) {
			formatNumber++;
			formatString += kgrid.client.workflow.Consts.PASEP + "="
					+ map.get(kgrid.client.workflow.Consts.PASEP);
		}
		if (map.containsKey(kgrid.client.workflow.Consts.PRSEP)) {
			formatNumber++;
			formatString += "," + kgrid.client.workflow.Consts.PRSEP + "="
					+ map.get(kgrid.client.workflow.Consts.PRSEP);
		}
		if (map.containsKey(kgrid.client.workflow.Consts.PUTOK)) {
			formatNumber++;
			formatString += "," + kgrid.client.workflow.Consts.PUTOK + "="
					+ map.get(kgrid.client.workflow.Consts.PUTOK);
		}
		if (formatNumber > 0) {
			if (formatString.startsWith(",")) {
				formatString = formatString.substring(1);
			}
		}
		if (formatNumber > 1) {
			size -= formatNumber - 1;
		}
		String[] content = new String[size];
		int i = 0;
		for (Map.Entry<String, String> e : map.entrySet()) {
			String k = e.getKey();
			String v = e.getValue();
			if (k.equals("Name")) {
				content[i] = getNameQuery(v);
			} else if (k.equals("Size")) {
				content[i] = getAccessQuery(v);
			}
			i++;
		}
		return content;
	}

	private String getNameQuery(String value) {
		return "Dataset[name=" + value + "]";
	}

	private String getAccessQuery(String value) {
		return "Access{Size=" + value + "}";
	}

	/**
	 * Returns the org.jdom.Element of the currently selected node.
	 */
	private Element getElementContent() {
		Object cell = workflow.getSelectedCell();
		if (cell instanceof WorkflowNode) {
			return ((WorkflowNode) cell).getElement();
		}
		return null;
	}

	/**
	 * Returns the org.jdom.Element of the currently selected node.
	 */
	public Element getElement() {
		Element e = getElementContent();
		List<?> children = e.getChildren();
		for (Object child : children) {
			Element data = (Element) child;
			String value = data.getAttributeValue("key");
			if (value.equals("description") || value.equals("metadata")) {
				// TODO xml bug: description or metadata tag? IGNORE
				// metadata
				if (!data.getChildren().isEmpty()) {
					return (Element) data.getChildren().get(0);
				}
			}
		}
		return null;
	}

}

class MyListModel extends AbstractListModel {
	private static final long serialVersionUID = 1L;
	String[] headers = {};

	public int getSize() {
		return headers.length;
	}

	public Object getElementAt(int index) {
		return headers[index];
	}

	public void setHeaders(LinkedList<Object[]> list) {
		headers = new String[list.size()];
		int i = 0;
		for (Object[] pair : list) {
			headers[i] = (String) pair[0];
			i++;
		}
	}
}
