/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.client.gui;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.JTextField;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class Settings {

	private static final int DEFAULT_PORT = 8443;
	private static JTextField epmsTextField;
	private static JTextField dasTextField;
	private static JTextField taasTextField;
	private final static String FILE = "settings.xml";

	private static String epmsHostname;
	private static String dasHostname;
	private static String taasHostname;

	private static int epmsPort;
	private static int dasPort;
	private static int taasPort;

	public Settings(JTextField e, JTextField d, JTextField t) {
		epmsTextField = e;
		dasTextField = d;
		taasTextField = t;
		loadFromFile();
	}

	public void saveToFile() {
		Document doc = new Document();
		Element settings = new Element("settings");

		Element epmsE = new Element("epms");
		epmsE.setText(epmsTextField.getText());
		settings.addContent(epmsE);

		Element dasE = new Element("das");
		dasE.setText(dasTextField.getText());
		settings.addContent(dasE);

		Element taasE = new Element("taas");
		taasE.setText(taasTextField.getText());
		settings.addContent(taasE);

		doc.addContent(settings);

		XMLOutputter outp = new XMLOutputter();
		outp.setFormat(Format.getPrettyFormat());

		try {
			outp.output(doc, new FileOutputStream(FILE));
		} catch (FileNotFoundException e) {
			JOptionPane.showMessageDialog(null, e, "FileNotFoundException",
					JOptionPane.ERROR_MESSAGE);
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, e, "IOException",
					JOptionPane.ERROR_MESSAGE);
		}
	}

	public static String getEPMSAddress() {
		epmsHostname = getAddress(epmsTextField);
		return epmsHostname;
	}

	public static int getEPMSPort() {
		epmsPort = getPort(epmsTextField);
		return epmsPort;
	}

	public static String getDASAddress() {
		dasHostname = getAddress(dasTextField);
		return dasHostname;
	}

	public static int getDASPort() {
		dasPort = getPort(dasTextField);
		return dasPort;
	}

	public static String getTAASAddress() {
		taasHostname = getAddress(taasTextField);
		return taasHostname;
	}

	public static int getTAASPort() {
		taasPort = getPort(taasTextField);
		return taasPort;
	}

	private void loadFromFile() {
		File file = new File(FILE);
		if (file.exists()) {
			SAXBuilder builder = new SAXBuilder();
			try {
				Document doc = builder.build(FILE);
				Element settingsE = doc.getRootElement();
				if (settingsE.getName().equals("settings")) {
					List<?> nodes = settingsE.getChildren();
					for (Object child : nodes) {
						Element e = (Element) child;

						String text = e.getText();
						if (text != null) {
							String eName = e.getName();
							if (eName.equals("epms")) {
								epmsTextField.setText(text);
							} else if (eName.equals("das")) {
								dasTextField.setText(text);
							} else if (eName.equals("taas")) {
								taasTextField.setText(text);
							} else {
								throw new JDOMException("Tag <" + eName
										+ "> unknown");
							}
						}
					}
				} else {
					throw new JDOMException("Tag <" + settingsE.getName()
							+ "> unknown");
				}
			} catch (JDOMException e) {
				JOptionPane.showMessageDialog(null, e, "JDOMException",
						JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			} catch (IOException e) {
				JOptionPane.showMessageDialog(null, e, "IOException",
						JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}
		}
		parseAddresses();
	}

	private void parseAddresses() {
		epmsHostname = getAddress(epmsTextField);
		epmsPort = getPort(epmsTextField);
		dasHostname = getAddress(dasTextField);
		dasPort = getPort(dasTextField);
		taasHostname = getAddress(taasTextField);
		taasPort = getPort(taasTextField);
	}

	private static String getAddress(JTextField textField) {
		String hostname = textField.getText();
		if (hostname.contains(":")) {
			hostname = hostname.substring(0, hostname.indexOf(":"));
		}
		return hostname;
	}

	private static int getPort(JTextField textField) {
		String hostname = textField.getText();
		if (hostname.contains(":")) {
			String portString = hostname.substring(hostname.indexOf(":") + 1);
			try {
				return Integer.parseInt(portString);
			} catch (NumberFormatException e) {
				throw new NumberFormatException("The port number " + portString
						+ "is not valid");
			}
		}
		return DEFAULT_PORT;
	}
}
