/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.client.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.LinkedList;
import java.util.StringTokenizer;

import javax.swing.AbstractAction;
import javax.swing.DefaultListModel;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JOptionPane;
import javax.swing.Timer;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

import kgrid.client.remote.DASTask;
import kgrid.client.remote.EPMSTask;
import kgrid.client.remote.GridProxy;
import kgrid.client.remote.TAASTask;
import kgrid.client.workflow.DatasetNode;
import kgrid.client.workflow.Edge;
import kgrid.client.workflow.ToolNode;
import kgrid.client.workflow.Workflow;
import kgrid.client.workflow.WorkflowNode;

import org.jdesktop.application.Action;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.Task;
import org.jdesktop.application.TaskMonitor;
import org.jdom.Element;
import org.jgraph.graph.PortView;

/**
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class View extends FrameView implements Consts {

	private static JFrame mainFrame;

	public View(SingleFrameApplication app) {
		super(app);
		initComponents();

		rolloverBorder = jButtonProcess.getBorder();

		// status bar initialization - message timeout, idle icon and busy
		// animation, etc
		int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
		messageTimer = new Timer(messageTimeout, new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				statusMessageLabel.setText("");
			}
		});
		messageTimer.setRepeats(false);
		int busyAnimationRate = resourceMap
				.getInteger("StatusBar.busyAnimationRate");
		for (int i = 0; i < busyIcons.length; i++) {
			busyIcons[i] = resourceMap
					.getIcon("StatusBar.busyIcons[" + i + "]");
		}
		busyIconTimer = new Timer(busyAnimationRate, new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
				statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
			}
		});
		idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
		statusAnimationLabel.setIcon(idleIcon);
		progressBar.setVisible(false);

		// connecting action tasks to status bar via TaskMonitor
		TaskMonitor taskMonitor = new TaskMonitor(getApplication().getContext());
		taskMonitor
				.addPropertyChangeListener(new java.beans.PropertyChangeListener() {

					public void propertyChange(
							java.beans.PropertyChangeEvent evt) {
						String propertyName = evt.getPropertyName();
						if ("started".equals(propertyName)) {
							if (!busyIconTimer.isRunning()) {
								statusAnimationLabel.setIcon(busyIcons[0]);
								busyIconIndex = 0;
								busyIconTimer.start();
							}
							progressBar.setVisible(true);
							progressBar.setIndeterminate(true);
						} else if ("done".equals(propertyName)) {
							busyIconTimer.stop();
							statusAnimationLabel.setIcon(idleIcon);
							progressBar.setVisible(false);
							progressBar.setValue(0);
						} else if ("message".equals(propertyName)) {
							String text = (String) (evt.getNewValue());
							statusMessageLabel.setText((text == null) ? ""
									: text);
							messageTimer.restart();
						} else if ("progress".equals(propertyName)) {
							int value = (Integer) (evt.getNewValue());
							progressBar.setVisible(true);
							progressBar.setIndeterminate(false);
							progressBar.setValue(value);
						}
					}
				});

		mainFrame = this.getFrame();
		mainFrame
				.setMinimumSize(new Dimension(FRAME_MINWIDTH, FRAME_MINHEIGHT));
		mainFrame.setPreferredSize(getStartingDimension());
		mainFrame.setMaximumSize(Toolkit.getDefaultToolkit().getScreenSize());
		mainFrame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				exitItemActionPerformed();
			}
		});
	}

	private Dimension getStartingDimension() {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		return new Dimension((int) screenSize.getWidth() - SHRINK_PIXELS,
				(int) screenSize.getHeight() - SHRINK_PIXELS);
	}

	@Action
	public void showAboutBox() {
		if (aboutBox == null) {
			JFrame mainFrame = App.getApplication().getMainFrame();
			aboutBox = new AboutBox(mainFrame);
			aboutBox.setLocationRelativeTo(mainFrame);
		}
		App.getApplication().show(aboutBox);
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	// @SuppressWarnings("unchecked")
	// <editor-fold defaultstate="collapsed"
	// desc="Generated Code">//GEN-BEGIN:initComponents
	private void initComponents() {

		mainPanel = new javax.swing.JPanel();
		jPropertiesPanel = new javax.swing.JPanel();

		workflow = new Workflow(this);
		propertiesPanel = new PropertiesPanel(workflow);

		jToolBar = new javax.swing.JToolBar();
		jRunButton = new javax.swing.JButton();
		jStopButton = new javax.swing.JButton();
		jResetButton = new javax.swing.JButton();
		jSeparator1 = new javax.swing.JToolBar.Separator();
		jButtonDataset = new javax.swing.JButton();
		jButtonProcess = new javax.swing.JButton();
		jButtonModel = new javax.swing.JButton();
		jButtonEdge = new javax.swing.JButton();
		jButtonDelete = new javax.swing.JButton();
		jButtonSelect = new javax.swing.JButton();
		jWrkFlwTabPane = new javax.swing.JTabbedPane();
		jPanel1 = new javax.swing.JPanel();
		jScrollPane2 = new javax.swing.JScrollPane(workflow);
		jResultsPanel = new javax.swing.JPanel();
		jScrollPane3 = new javax.swing.JScrollPane();
		jResultsList = new javax.swing.JList();
		searchButton = new javax.swing.JButton();
		menuBar = new javax.swing.JMenuBar();
		newItem = new javax.swing.JMenuItem();
		openItem = new javax.swing.JMenuItem();
		saveItem = new javax.swing.JMenuItem();
		saveAsItem = new javax.swing.JMenuItem();
		exitMenuItem = new javax.swing.JMenuItem();
		snapMenuItem = new javax.swing.JCheckBoxMenuItem();
		settingsMenuItem = new javax.swing.JMenuItem();
		javax.swing.JMenu helpMenu = new javax.swing.JMenu();
		javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
		statusPanel = new javax.swing.JPanel();
		javax.swing.JSeparator statusPanelSeparator = new javax.swing.JSeparator();
		statusMessageLabel = new javax.swing.JLabel();
		statusAnimationLabel = new javax.swing.JLabel();
		progressBar = new javax.swing.JProgressBar();

		mainPanel.setName("mainPanel"); // NOI18N

		org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application
				.getInstance(kgrid.client.gui.App.class).getContext()
				.getResourceMap(View.class);
		propertiesBorder = javax.swing.BorderFactory
				.createTitledBorder(resourceMap
						.getString("jPropertiesPanel.border.title"));
		jPropertiesPanel.setBorder(propertiesBorder); // NOI18N
		jPropertiesPanel.setName("jPropertiesPanel"); // NOI18N

		propertiesPanel.setName("jPropertiesPanel"); // NOI18N

		javax.swing.GroupLayout jSearchPanelLayout = new javax.swing.GroupLayout(
				jPropertiesPanel);
		jPropertiesPanel.setLayout(jSearchPanelLayout);
		jSearchPanelLayout.setHorizontalGroup(jSearchPanelLayout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(propertiesPanel,
						javax.swing.GroupLayout.DEFAULT_SIZE, 286,
						Short.MAX_VALUE));
		jSearchPanelLayout.setVerticalGroup(jSearchPanelLayout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(propertiesPanel,
						javax.swing.GroupLayout.PREFERRED_SIZE, 205,
						Short.MAX_VALUE));

		jToolBar.setFloatable(false);
		jToolBar.setRollover(true);
		jToolBar.setName("jToolBar"); // NOI18N

		javax.swing.ActionMap actionMap = org.jdesktop.application.Application
				.getInstance(kgrid.client.gui.App.class).getContext()
				.getActionMap(View.class, this);
		jRunButton.setAction(actionMap.get("submitApplication")); // NOI18N
		jRunButton.setIcon(resourceMap.getIcon("jRunButton.icon")); // NOI18N
		jRunButton.setToolTipText(resourceMap
				.getString("jRunButton.toolTipText")); // NOI18N
		jRunButton.setFocusable(false);
		jRunButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		jRunButton.setMargin(new java.awt.Insets(1, 1, 1, 1));
		jRunButton.setMaximumSize(new java.awt.Dimension(34, 34));
		jRunButton.setMinimumSize(new java.awt.Dimension(34, 34));
		jRunButton.setName("jRunButton"); // NOI18N
		jRunButton.setPreferredSize(new java.awt.Dimension(35, 35));
		jRunButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
		// jRunButton.addActionListener(new java.awt.event.ActionListener() {
		// public void actionPerformed(java.awt.event.ActionEvent evt) {
		// jButtonRunActionPerformed(evt);
		// }
		// });

		jToolBar.add(jRunButton);

		jStopButton.setIcon(resourceMap.getIcon("jStopButton.icon")); // NOI18N
		jStopButton.setToolTipText(resourceMap
				.getString("jStopButton.toolTipText")); // NOI18N
		jStopButton.setFocusable(false);
		jStopButton
				.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		jStopButton.setMargin(new java.awt.Insets(1, 1, 1, 1));
		jStopButton.setMaximumSize(new java.awt.Dimension(34, 34));
		jStopButton.setMinimumSize(new java.awt.Dimension(34, 34));
		jStopButton.setName("jStopButton"); // NOI18N
		jStopButton.setPreferredSize(new java.awt.Dimension(35, 35));
		jStopButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
		jStopButton.setEnabled(false);
		jToolBar.add(jStopButton);

		jResetButton.setIcon(resourceMap.getIcon("jResetButton.icon")); // NOI18N
		jResetButton.setToolTipText(resourceMap
				.getString("jResetButton.toolTipText")); // NOI18N
		jResetButton.setFocusable(false);
		jResetButton
				.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		jResetButton.setMargin(new java.awt.Insets(1, 1, 1, 1));
		jResetButton.setMaximumSize(new java.awt.Dimension(34, 34));
		jResetButton.setMinimumSize(new java.awt.Dimension(34, 34));
		jResetButton.setName("jResetButton"); // NOI18N
		jResetButton.setPreferredSize(new java.awt.Dimension(35, 35));
		jResetButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
		jResetButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonResetActionPerformed(evt);
			}
		});
		jToolBar.add(jResetButton);

		jSeparator1.setName("jSeparator1"); // NOI18N
		jToolBar.add(jSeparator1);

		jButtonDataset.setIcon(resourceMap.getIcon("jButtonDataset.icon")); // NOI18N
		jButtonDataset.setToolTipText(resourceMap
				.getString("jButtonDataset.toolTipText")); // NOI18N
		jButtonDataset.setBorder(selectedBorder);
		jButtonDataset.setDisabledIcon(resourceMap
				.getIcon("jButtonDataset.disabledIcon")); // NOI18N
		jButtonDataset.setEnabled(false);
		jButtonDataset.setFocusable(false);
		jButtonDataset
				.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		jButtonDataset.setMargin(new java.awt.Insets(1, 1, 1, 1));
		jButtonDataset.setMaximumSize(new java.awt.Dimension(34, 34));
		jButtonDataset.setMinimumSize(new java.awt.Dimension(34, 34));
		jButtonDataset.setName("jButtonDataset"); // NOI18N
		jButtonDataset.setPreferredSize(new java.awt.Dimension(35, 35));
		jButtonDataset
				.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
		jButtonDataset.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonDatasetActionPerformed(evt);
			}
		});
		jToolBar.add(jButtonDataset);

		jButtonProcess.setIcon(resourceMap.getIcon("jButtonProcess.icon")); // NOI18N
		jButtonProcess.setToolTipText(resourceMap
				.getString("jButtonProcess.toolTipText")); // NOI18N
		jButtonProcess.setDisabledIcon(resourceMap
				.getIcon("jButtonProcess.disabledIcon")); // NOI18N
		jButtonProcess.setFocusable(false);
		jButtonProcess
				.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		jButtonProcess.setMargin(new java.awt.Insets(1, 1, 1, 1));
		jButtonProcess.setMaximumSize(new java.awt.Dimension(34, 34));
		jButtonProcess.setMinimumSize(new java.awt.Dimension(34, 34));
		jButtonProcess.setName("jButtonProcess"); // NOI18N
		jButtonProcess.setPreferredSize(new java.awt.Dimension(35, 35));
		jButtonProcess
				.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
		jButtonProcess.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonProcessActionPerformed(evt);
			}
		});
		jToolBar.add(jButtonProcess);

		jButtonModel.setIcon(resourceMap.getIcon("jButtonModel.icon")); // NOI18N
		jButtonModel.setToolTipText(resourceMap
				.getString("jButtonModel.toolTipText")); // NOI18N
		jButtonModel.setDisabledIcon(resourceMap
				.getIcon("jButtonModel.disabledIcon")); // NOI18N
		jButtonModel.setFocusable(false);
		jButtonModel
				.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		jButtonModel.setMargin(new java.awt.Insets(1, 1, 1, 1));
		jButtonModel.setMaximumSize(new java.awt.Dimension(34, 34));
		jButtonModel.setMinimumSize(new java.awt.Dimension(34, 34));
		jButtonModel.setName("jButtonModel"); // NOI18N
		jButtonModel.setPreferredSize(new java.awt.Dimension(35, 35));
		jButtonModel.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
		jButtonModel.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonModelActionPerformed(evt);
			}
		});
		jToolBar.add(jButtonModel);

		jButtonEdge.setIcon(resourceMap.getIcon("jButtonEdge.icon")); // NOI18N
		jButtonEdge.setToolTipText(resourceMap
				.getString("jButtonEdge.toolTipText")); // NOI18N
		jButtonEdge.setDisabledIcon(resourceMap
				.getIcon("jButtonEdge.disabledIcon")); // NOI18N
		jButtonEdge.setFocusable(false);
		jButtonEdge
				.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		jButtonEdge.setMargin(new java.awt.Insets(1, 1, 1, 1));
		jButtonEdge.setMaximumSize(new java.awt.Dimension(34, 34));
		jButtonEdge.setMinimumSize(new java.awt.Dimension(34, 34));
		jButtonEdge.setName("jButtonEdge"); // NOI18N
		jButtonEdge.setPreferredSize(new java.awt.Dimension(35, 35));
		jButtonEdge.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
		jButtonEdge.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonEdgeActionPerformed(evt);
			}
		});
		jToolBar.add(jButtonEdge);

		jButtonSelect.setIcon(resourceMap.getIcon("jButtonSelect.icon")); // NOI18N
		jButtonSelect.setToolTipText(resourceMap
				.getString("jButtonSelect.toolTipText")); // NOI18N
		jButtonSelect.setDisabledIcon(resourceMap
				.getIcon("jButtonSelect.disabledIcon")); // NOI18N
		jButtonSelect.setFocusable(false);
		jButtonSelect
				.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		jButtonSelect.setMargin(new java.awt.Insets(1, 1, 1, 1));
		jButtonSelect.setMaximumSize(new java.awt.Dimension(34, 34));
		jButtonSelect.setMinimumSize(new java.awt.Dimension(34, 34));
		jButtonSelect.setName("jButtonSelect"); // NOI18N
		jButtonSelect.setPreferredSize(new java.awt.Dimension(35, 35));
		jButtonSelect
				.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
		jButtonSelect.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonSelectActionPerformed(evt);
			}
		});
		jToolBar.add(jButtonSelect);

		jButtonDelete.setIcon(resourceMap.getIcon("jButtonDelete.icon")); // NOI18N
		jButtonDelete.setToolTipText(resourceMap
				.getString("jButtonDelete.toolTipText")); // NOI18N
		jButtonDelete.setDisabledIcon(resourceMap
				.getIcon("jButtonDelete.disabledIcon")); // NOI18N
		jButtonDelete.setFocusable(false);
		jButtonDelete
				.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		jButtonDelete.setMargin(new java.awt.Insets(1, 1, 1, 1));
		jButtonDelete.setMaximumSize(new java.awt.Dimension(34, 34));
		jButtonDelete.setMinimumSize(new java.awt.Dimension(34, 34));
		jButtonDelete.setName("jButtonDelete"); // NOI18N
		jButtonDelete.setPreferredSize(new java.awt.Dimension(35, 35));
		jButtonDelete
				.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
		jButtonDelete.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonDeleteActionPerformed(evt);
			}
		});
		jToolBar.add(jButtonDelete);

		jWrkFlwTabPane.setName("jWrkFlwTabPane"); // NOI18N

		jPanel1.setName("jPanel1"); // NOI18N

		jScrollPane2.setBackground(resourceMap
				.getColor("jScrollPane2.background")); // NOI18N
		jScrollPane2.setName("jScrollPane2"); // NOI18N

		javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(
				jPanel1);
		jPanel1.setLayout(jPanel1Layout);
		jPanel1Layout.setHorizontalGroup(jPanel1Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				jScrollPane2, WRKFLW_PANEL_MINWIDTH, 6970, Short.MAX_VALUE));
		jPanel1Layout.setVerticalGroup(jPanel1Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				jScrollPane2, WRKFLW_PANEL_MINHEIGHT, 5210, Short.MAX_VALUE));

		jWrkFlwTabPane.addTab(Workflow.DEFAULT_FILE_NAME, jPanel1); // NOI18N

		jResultsPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder(resourceMap
						.getString("jResultsPanel.border.title"))); // NOI18N
		jResultsPanel.setName("jResultsPanel"); // NOI18N

		jScrollPane3.setName("jScrollPane3"); // NOI18N

		jResultsList.setName("jResultsList"); // NOI18N
		jScrollPane3.setViewportView(jResultsList);

		javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(
				jResultsPanel);
		jResultsPanel.setLayout(jPanel2Layout);
		jPanel2Layout.setHorizontalGroup(jPanel2Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 286,
				Short.MAX_VALUE));
		jPanel2Layout.setVerticalGroup(jPanel2Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 212,
				Short.MAX_VALUE));

		searchButton.setAction(actionMap.get("searchResource")); // NOI18N
		searchButton.setIcon(resourceMap.getIcon("searchButton.icon")); // NOI18N
		searchButton.setText(resourceMap.getString("searchButton.text")); // NOI18N
		searchButton.setName("searchButton"); // NOI18N
		searchButton.setEnabled(false);

		javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(
				mainPanel);
		mainPanel.setLayout(mainPanelLayout);
		mainPanelLayout
				.setHorizontalGroup(mainPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								mainPanelLayout
										.createSequentialGroup()
										.addGroup(
												mainPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.TRAILING,
																false)
														.addComponent(
																jToolBar,
																javax.swing.GroupLayout.Alignment.LEADING,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addGroup(
																javax.swing.GroupLayout.Alignment.LEADING,
																mainPanelLayout
																		.createSequentialGroup()
																		.addContainerGap()
																		.addComponent(
																				searchButton))
														.addComponent(
																jPropertiesPanel,
																javax.swing.GroupLayout.Alignment.LEADING,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addComponent(
																jResultsPanel,
																javax.swing.GroupLayout.Alignment.LEADING,
																0,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(
												jWrkFlwTabPane,
												WRKFLW_TABPANE_MINWIDTH,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE)));
		mainPanelLayout
				.setVerticalGroup(mainPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								mainPanelLayout
										.createSequentialGroup()
										.addGroup(
												mainPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addGroup(
																mainPanelLayout
																		.createSequentialGroup()
																		.addComponent(
																				jToolBar,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				javax.swing.GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				jPropertiesPanel,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				Short.MAX_VALUE)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				searchButton)
																		.addGap(1,
																				1,
																				1)
																		.addComponent(
																				jResultsPanel,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				javax.swing.GroupLayout.PREFERRED_SIZE))
														.addComponent(
																jWrkFlwTabPane,
																WRKFLW_TABPANE_MINHEIGHT,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE))));

		jPropertiesPanel
				.getAccessibleContext()
				.setAccessibleName(
						resourceMap
								.getString("jPropertiesPanel.AccessibleContext.accessibleName")); // NOI18N
		jWrkFlwTabPane
				.getAccessibleContext()
				.setAccessibleName(
						resourceMap
								.getString("jWrkFlwTabPane.AccessibleContext.accessibleName")); // NOI18N

		menuBar.setName("menuBar"); // NOI18N

		fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
		fileMenu.setName("fileMenu"); // NOI18N

		newItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_N,
				java.awt.event.InputEvent.CTRL_MASK));
		newItem.setIcon(resourceMap.getIcon("newItem.icon")); // NOI18N
		newItem.setText(resourceMap.getString("newItem.text")); // NOI18N
		newItem.setName("newItem"); // NOI18N
		newItem.setEnabled(false);
		newItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				newItemActionPerformed(evt);
			}
		});
		fileMenu.add(newItem);

		openItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_O,
				java.awt.event.InputEvent.CTRL_MASK));
		openItem.setIcon(resourceMap.getIcon("openItem.icon")); // NOI18N
		openItem.setText(resourceMap.getString("openItem.text")); // NOI18N
		openItem.setName("openItem"); // NOI18N
		openItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				openItemActionPerformed(evt);
			}
		});
		fileMenu.add(openItem);

		saveItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_S,
				java.awt.event.InputEvent.CTRL_MASK));
		saveItem.setIcon(resourceMap.getIcon("saveItem.icon")); // NOI18N
		saveItem.setText(resourceMap.getString("saveItem.text")); // NOI18N
		saveItem.setName("saveItem"); // NOI18N
		saveItem.setEnabled(false);
		saveItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				saveActionPerformed(evt);
			}
		});
		fileMenu.add(saveItem);

		saveAsItem.setIcon(resourceMap.getIcon("saveAsItem.icon")); // NOI18N
		saveAsItem.setText(resourceMap.getString("saveAsItem.text")); // NOI18N
		saveAsItem.setName("saveAsItem"); // NOI18N
		saveAsItem.setEnabled(true);
		saveAsItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				saveAsActionPerformed(evt);
			}
		});
		fileMenu.add(saveAsItem);

		// exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
		exitMenuItem.setIcon(resourceMap.getIcon("exitItem.icon")); // NOI18N
		exitMenuItem.setText(resourceMap.getString("exitItem.text")); // NOI18N
		exitMenuItem.setName("exitMenuItem"); // NOI18N
		exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				exitItemActionPerformed();
			}
		});
		fileMenu.addSeparator();
		fileMenu.add(exitMenuItem);

		menuBar.add(fileMenu);

		optionsMenu.setText(resourceMap.getString("optionsMenu.text")); // NOI18N
		optionsMenu.setName("optionsMenu"); // NOI18N

		snapMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_G,
				java.awt.event.InputEvent.CTRL_MASK));
		snapMenuItem.setText(resourceMap.getString("snapMenuItem.text")); // NOI18N
		snapMenuItem.setName("snapMenuItem"); // NOI18N
		snapMenuItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				snapItemActionPerformed(evt);
			}
		});
		optionsMenu.add(snapMenuItem);

		settingsMenuItem.setIcon(resourceMap.getIcon("settingsItem.icon")); // NOI18N
		settingsMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_O,
				java.awt.event.InputEvent.CTRL_MASK));
		settingsMenuItem
				.setText(resourceMap.getString("settingsMenuItem.text")); // NOI18N
		settingsMenuItem.setName("settingsMenuItem"); // NOI18N
		settingsMenuItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				settingsItemActionPerformed(evt);
			}

		});
		optionsMenu.add(settingsMenuItem);

		menuBar.add(optionsMenu);

		resultsMenu.setText(resourceMap.getString("resMenu.text")); // NOI18N
		resultsMenu.setName("resMenu"); // NOI18N
		resultsMenu.addChangeListener(new ResultsMenuListener(resultsMenu,
				mainFrame));

		menuBar.add(resultsMenu);

		// resultsMenu.setAction(arg0);

		helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
		helpMenu.setName("helpMenu"); // NOI18N

		aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
		aboutMenuItem.setIcon(resourceMap.getIcon("aboutItem.icon"));
		aboutMenuItem.setName("aboutMenuItem"); // NOI18N
		helpMenu.add(aboutMenuItem);

		menuBar.add(helpMenu);

		statusPanel.setName("statusPanel"); // NOI18N

		statusPanelSeparator.setName("statusPanelSeparator"); // NOI18N

		statusMessageLabel.setName("statusMessageLabel"); // NOI18N

		statusAnimationLabel
				.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
		statusAnimationLabel.setName("statusAnimationLabel"); // NOI18N

		progressBar.setName("progressBar"); // NOI18N

		javax.swing.GroupLayout statusPanelLayout = new javax.swing.GroupLayout(
				statusPanel);
		statusPanel.setLayout(statusPanelLayout);
		statusPanelLayout
				.setHorizontalGroup(statusPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addComponent(statusPanelSeparator,
								javax.swing.GroupLayout.DEFAULT_SIZE, 1236,
								Short.MAX_VALUE)
						.addGroup(
								statusPanelLayout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(statusMessageLabel)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED,
												1066, Short.MAX_VALUE)
										.addComponent(
												progressBar,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(statusAnimationLabel)
										.addContainerGap()));
		statusPanelLayout
				.setVerticalGroup(statusPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								statusPanelLayout
										.createSequentialGroup()
										.addComponent(
												statusPanelSeparator,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												2,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)
										.addGroup(
												statusPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																statusMessageLabel)
														.addComponent(
																statusAnimationLabel)
														.addComponent(
																progressBar,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addGap(3, 3, 3)));

		setComponent(mainPanel);
		setMenuBar(menuBar);
		setStatusBar(statusPanel);
	}// </editor-fold>//GEN-END:initComponents

	public PropertiesPanel getPropertiesPanel() {
		return propertiesPanel;
	}

	private void addNewDesignBoard() {
		tabsCount++;
		javax.swing.JPanel jNewPanel = new javax.swing.JPanel();
		jNewPanel.setName("jPanel1"); // NOI18N

		jScrollPane2.setBackground(resourceMap
				.getColor("jScrollPane2.background")); // NOI18N
		jScrollPane2.setName("jScrollPane2"); // NOI18N

		javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(
				jNewPanel);
		jNewPanel.setLayout(jPanel1Layout);
		jPanel1Layout.setHorizontalGroup(jPanel1Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 594,
				Short.MAX_VALUE));
		jPanel1Layout.setVerticalGroup(jPanel1Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				jPanel1Layout
						.createSequentialGroup()
						.addComponent(jScrollPane2,
								javax.swing.GroupLayout.DEFAULT_SIZE, 537,
								Short.MAX_VALUE).addGap(12, 12, 12)));

		jWrkFlwTabPane.addTab("Untitled" + tabsCount, jNewPanel); // NOI18N

	}

	private void jButtonDeleteActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonDeleteActionPerformed
		setEditingStatus(EditingStatusType.DELETE);
	}// GEN-LAST:event_jButtonDeleteActionPerformed

	private void jButtonSelectActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonDeleteActionPerformed
		setEditingStatus(EditingStatusType.SELECT);
	}// GEN-LAST:event_jButtonDeleteActionPerformed

	private void jButtonEdgeActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonEdgeActionPerformed
		setEditingStatus(EditingStatusType.EDGE);
	}// GEN-LAST:event_jButtonEdgeActionPerformed

	private void jButtonModelActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonModelActionPerformed
		setEditingStatus(EditingStatusType.MODEL);
	}// GEN-LAST:event_jButtonModelActionPerformed

	private void jButtonProcessActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonProcessActionPerformed
		setEditingStatus(EditingStatusType.TOOL);
	}// GEN-LAST:event_jButtonProcessActionPerformed

	private void jButtonDatasetActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonDatasetActionPerformed
		setEditingStatus(EditingStatusType.DATASET);
	}// GEN-LAST:event_jButtonDatasetActionPerformed

	private void jButtonResetActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonDatasetActionPerformed
		workflow.resetNodesStatuses();
	}// GEN-LAST:event_jButtonDatasetActionPerformed

	private void newItemActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_newItemActionPerformed
		// TODO KGClient: addNewDesignBoard();
		if (operationConfirmed()) {
			newItem.setEnabled(false);
			openItem.setEnabled(true);
			saveItem.setEnabled(false);
			propertiesPanel.clear();
			workflow.clear();
			workflow.setChanged(false);
		}

	}// GEN-LAST:event_newItemActionPerformed

	private void openItemActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_newItemActionPerformed
		if (operationConfirmed()) {
			int result = chooser.showOpenDialog(mainFrame);

			if (result == JFileChooser.APPROVE_OPTION) {
				String newFileName = chooser.getSelectedFile().getPath();
				workflow.clear();
				workflow.loadFromFile(newFileName);
				workflow.setChanged(false);
				newItem.setEnabled(true);
				openItem.setEnabled(true);
				saveItem.setEnabled(false);
				setNodeProperties(null);
				mainFrame.setTitle(resourceMap.getString("Application.title"));
				getComponent().repaint();
			}
		}
	}// GEN-LAST:event_openItemActionPerformed

	private void exitItemActionPerformed() {
		String question;
		int messageType;

		if (workflow.hasChanged()) {
			question = "Exit discarding the changes?";
			messageType = JOptionPane.WARNING_MESSAGE;
		} else {
			question = "Exit " + resourceMap.getString("Application.title")
					+ "?";
			messageType = JOptionPane.QUESTION_MESSAGE;
		}

		int selection = JOptionPane.showConfirmDialog(getComponent(), question,
				"Confirm exit", JOptionPane.OK_CANCEL_OPTION, messageType);

		if (selection == JOptionPane.OK_OPTION) {
			System.exit(0);
		}
	}

	private boolean operationConfirmed() {
		boolean confirmed = false;
		if (workflow.hasChanged()) {
			int selection = JOptionPane.showConfirmDialog(getComponent(),
					"Open a new workflow discarding the changes?",
					"Open workflow", JOptionPane.OK_CANCEL_OPTION,
					JOptionPane.WARNING_MESSAGE);
			if (selection == JOptionPane.OK_OPTION) {
				confirmed = true;
			}
		} else {
			confirmed = true;
		}
		return confirmed;
	}

	private void resetEditingButtonBorder() {
		switch (editingStatus) {
		case DATASET:
			jButtonDataset.setBorder(rolloverBorder);
			jButtonDataset.setEnabled(true);
			break;
		case TOOL:
			jButtonProcess.setBorder(rolloverBorder);
			jButtonProcess.setEnabled(true);
			break;
		case MODEL:
			jButtonModel.setBorder(rolloverBorder);
			jButtonModel.setEnabled(true);
			break;
		case EDGE:
			jButtonEdge.setBorder(rolloverBorder);
			jButtonEdge.setEnabled(true);
			break;
		case DELETE:
			jButtonDelete.setBorder(rolloverBorder);
			jButtonDelete.setEnabled(true);
			break;
		case SELECT:
			jButtonSelect.setBorder(rolloverBorder);
			jButtonSelect.setEnabled(true);
		}
	}

	private void setEditingStatus(EditingStatusType s) {
		if (s == EditingStatusType.EDGE) {
			PortView.SIZE = 34;
			workflow.setPortsVisible(true);
		} else {
			PortView.SIZE = 0;
			workflow.setPortsVisible(false);
		}
		setEditingButton(s);
		editingStatus = s;
		if (s == EditingStatusType.SELECT) {
			workflow.enableSelection(true);
		} else {
			workflow.enableSelection(false);
		}
	}

	public void switchEditingStatus() {
		switch (editingStatus) {
		case DATASET:
			setEditingStatus(EditingStatusType.TOOL);
			break;
		case TOOL:
			setEditingStatus(EditingStatusType.MODEL);
			break;
		case MODEL:
			setEditingStatus(EditingStatusType.EDGE);
			setNodeProperties(null);
			break;
		case EDGE:
			setEditingStatus(EditingStatusType.SELECT);
			setNodeProperties(null);
			break;
		case SELECT:
			setEditingStatus(EditingStatusType.DATASET);
			setNodeProperties(null);
			break;
		case DELETE:
			setEditingStatus(EditingStatusType.DATASET);
			setNodeProperties(null);

		}
	}

	public void switchBackEditingStatus() {
		switch (editingStatus) {
		case DATASET:
			setEditingStatus(EditingStatusType.SELECT);
			setNodeProperties(null);
			break;
		case TOOL:
			setEditingStatus(EditingStatusType.DATASET);
			break;
		case MODEL:
			setEditingStatus(EditingStatusType.TOOL);
			break;
		case EDGE:
			setEditingStatus(EditingStatusType.MODEL);
			setNodeProperties(null);
			break;
		case SELECT:
			setEditingStatus(EditingStatusType.EDGE);
			setNodeProperties(null);
			break;
		case DELETE:
			setEditingStatus(EditingStatusType.SELECT);
			setNodeProperties(null);

		}
	}

	private void setEditingButton(EditingStatusType newStatus) {
		resetEditingButtonBorder();
		switch (newStatus) {
		case DATASET:
			jButtonDataset.setBorder(selectedBorder);
			jButtonDataset.setEnabled(false);
			break;
		case TOOL:
			jButtonProcess.setBorder(selectedBorder);
			jButtonProcess.setEnabled(false);
			break;
		case MODEL:
			jButtonModel.setBorder(selectedBorder);
			jButtonModel.setEnabled(false);
			break;
		case EDGE:
			jButtonEdge.setBorder(selectedBorder);
			jButtonEdge.setEnabled(false);
			break;
		case DELETE:
			jButtonDelete.setBorder(selectedBorder);
			jButtonDelete.setEnabled(false);
			break;
		case SELECT:
			jButtonSelect.setBorder(selectedBorder);
			jButtonSelect.setEnabled(false);
		}
	}

	@Action
	public Task<?, ?> searchResource() {
		return new SearchResourceTask(getApplication(), this);
	}

	private class SearchResourceTask extends
			org.jdesktop.application.Task<Object, Void> {

		private View view;

		SearchResourceTask(org.jdesktop.application.Application app, View view) {
			// Runs on the EDT. Copy GUI state that
			// doInBackground() depends on from parameters
			// to SearchResourceTask fields, here.
			super(app);
			this.view = view;
		}

		@Override
		protected Object doInBackground() {

			if (System.getProperty("GLOBUS_LOCATION") == null) {
				String message = "Environment variable GLOBUS_LOCATION is not defined. Try starting the JVM with the \"-DGLOBUS_LOCATION=$GLOBUS_LOCATION\" switch.";
				JOptionPane.showMessageDialog(mainFrame, message,
						"GLOBUS_LOCATION not set", JOptionPane.ERROR_MESSAGE);
				return null;
			}

			DefaultListModel model = new DefaultListModel();
			jResultsList.setModel(model);

			LinkedList<String> list = new LinkedList<String>();

			GridProxy.init();
			if (!GridProxy.isValid()) {
				String message = "Cannot submit application: credentials expired.\nGrid Proxy certificate required.";
				Utils.print(message);
				JOptionPane.showMessageDialog(mainFrame, message,
						"GSSException", JOptionPane.ERROR_MESSAGE);
				return null;
			}
			try {

				Object selectedNode = workflow.getSelectedCell();
				if (selectedNode instanceof DatasetNode) {
					if (Settings.getDASAddress().isEmpty()) {
						String message = "Please set the DAS host address in Options/Settings.";
						Utils.print(message);
						JOptionPane.showMessageDialog(mainFrame, message,
								"Error", JOptionPane.ERROR_MESSAGE);
						return null;
					}
					Element datasetElement = propertiesPanel.getElement();
					if (datasetElement != null) {
						DASTask task = new DASTask(datasetElement, view);
						task.invoke();
						String result = task.getResult();
						Utils.print("task.getResult() = " + result);
						StringTokenizer st = new StringTokenizer(result);
						while (st.hasMoreTokens()) {
							String token = st.nextToken(" ");
							if (token
									.startsWith(kgrid.client.workflow.Consts.KDSURL_PREFIX)) {
								list.add(token);
							}
						}
						if (list.isEmpty()) {
							JOptionPane.showMessageDialog(mainFrame,
									"No dataset found matching the criteria.",
									"Empty result",
									JOptionPane.INFORMATION_MESSAGE);
						}
					} else {
						JOptionPane
								.showMessageDialog(
										mainFrame,
										"At least one search parameter must be filled in.",
										"Error", JOptionPane.ERROR_MESSAGE);
					}
				} else if (selectedNode instanceof ToolNode) {
					if (Settings.getTAASAddress().isEmpty()) {
						String message = "Please set the TAAS host address in Options/Settings.";
						Utils.print(message);
						JOptionPane.showMessageDialog(mainFrame, message,
								"Error", JOptionPane.ERROR_MESSAGE);
						return null;
					}
					Element toolElement = propertiesPanel.getElement();
					if (toolElement != null) {
						TAASTask task = new TAASTask(toolElement, view);
						task.invoke();
						String result = task.getResult();
						Utils.print("task.getResult() = " + result);
						StringTokenizer st = new StringTokenizer(result);
						while (st.hasMoreTokens()) {
							String token = st.nextToken(" ");
							if (token
									.startsWith(kgrid.client.workflow.Consts.KDSURL_PREFIX)) {
								list.add(token);
							}
						}
						if (list.isEmpty()) {
							JOptionPane.showMessageDialog(mainFrame,
									"No dataset found matching the criteria.",
									"Empty result",
									JOptionPane.INFORMATION_MESSAGE);
						}
					} else {
						JOptionPane
								.showMessageDialog(
										mainFrame,
										"At least one search parameter must be filled in.",
										"Error", JOptionPane.ERROR_MESSAGE);
					}
				}
			} catch (Exception e) {
				String exMessage = e.toString();
				int endPos = exMessage.indexOf(":");
				String message = exMessage.substring(endPos + 2);
				if (message.contains("Connection timed out")) {
					message += "\nThe connection may be blocked\nby the client or the EPMS host firewall.";
				} else if (message.contains("Connection refused")) {
					message += "\nThe Globus container at "
							+ Settings.getEPMSAddress() + ":"
							+ Settings.getEPMSPort()
							+ " may not have been started.";
				} else if (message.contains("No route to host ")) {
					message += Settings.getEPMSAddress() + ":"
							+ Settings.getEPMSPort()
							+ " is unreachable (it may be down).";
				}
				String error = exMessage;
				int startPos = exMessage.lastIndexOf(".");
				if (0 <= startPos && startPos <= endPos) {
					error = exMessage.substring(startPos + 1, endPos);
				}
				Utils.print(exMessage);
				// TODO remove this printStackTrace when not debugging
				e.printStackTrace();
				JOptionPane.showMessageDialog(mainFrame, message, error,
						JOptionPane.ERROR_MESSAGE);
			}

			int i = 0;
			for (String s : list) {
				model.add(i++, s);
			}

			return null; // return your result
		}

		@Override
		protected void succeeded(Object result) {
			// Runs on the EDT. Update the GUI based on
			// the result computed by doInBackground().
		}
	}

	@Action
	public Task<?, ?> submitApplication() {
		workflow.resetNodesStatuses();
		return new submitApplicationTask(getApplication());
	}

	private class submitApplicationTask extends
			org.jdesktop.application.Task<Object, Void> {

		submitApplicationTask(org.jdesktop.application.Application app) {
			// Runs on the EDT. Copy GUI state that
			// doInBackground() depends on from parameters
			// to SearchDatasetTask fields, here.
			super(app);
		}

		@Override
		protected Object doInBackground() throws InterruptedException {
			jRunButton.setEnabled(false);
			jStopButton.setEnabled(true);
			jButtonDataset.setEnabled(false);
			jButtonDelete.setEnabled(false);
			jButtonSelect.setEnabled(false);
			jButtonEdge.setEnabled(false);
			jButtonProcess.setEnabled(false);
			jButtonModel.setEnabled(false);
			// TODO DIS3GNO Disable workflow modification during the execution
			if (workflow.isEmpty()) {
				String message = "Cannot submit application: the workflow is empty.";
				Utils.print(message);
				JOptionPane.showMessageDialog(mainFrame, message, "Error",
						JOptionPane.ERROR_MESSAGE);
				return null;
			}
			if (Settings.getEPMSAddress().isEmpty()) {
				String message = "Please set the EPMS host address in Options/Settings";
				Utils.print(message);
				JOptionPane.showMessageDialog(mainFrame, message, "Error",
						JOptionPane.ERROR_MESSAGE);
				return null;
			}
			if (System.getProperty("GLOBUS_LOCATION") == null) {
				String message = "Environment variable GLOBUS_LOCATION is not defined. Try starting the JVM with the \"-DGLOBUS_LOCATION=$GLOBUS_LOCATION\" switch.";
				JOptionPane.showMessageDialog(mainFrame, message,
						"GLOBUS_LOCATION not set", JOptionPane.ERROR_MESSAGE);
				return null;
			}
			String emptyNodesErrorMessage = workflow
					.getEmptyNodesErrorMessages();
			if (!emptyNodesErrorMessage.isEmpty()) {
				JOptionPane.showMessageDialog(mainFrame,
						emptyNodesErrorMessage, "Some nodes are empty",
						JOptionPane.ERROR_MESSAGE);
				return null;
			}
			GridProxy.init();
			if (!GridProxy.isValid()) {
				String message = "Cannot submit application: credentials expired.\nGrid Proxy certificate required.";
				Utils.print(message);
				JOptionPane.showMessageDialog(mainFrame, message,
						"GSSException", JOptionPane.ERROR_MESSAGE);
				return null;
			}
			workflow.start();
			return null; // return your result
		}

		@Override
		protected void succeeded(Object result) {
			// Runs on the EDT. Update the GUI based on
			// the result computed by doInBackground().
			jRunButton.setEnabled(true);
			jStopButton.setEnabled(false);
			jButtonDataset.setEnabled(true);
			jButtonDelete.setEnabled(true);
			jButtonSelect.setEnabled(true);
			jButtonEdge.setEnabled(true);
			jButtonProcess.setEnabled(true);
			jButtonModel.setEnabled(true);
		}
	}

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JButton jRunButton;
	private javax.swing.JButton jStopButton;
	private javax.swing.JButton jResetButton;
	private javax.swing.JButton jButtonDataset;
	private javax.swing.JButton jButtonDelete;
	private javax.swing.JButton jButtonSelect;
	private javax.swing.JButton jButtonEdge;
	private javax.swing.JButton jButtonProcess;
	private javax.swing.JButton jButtonModel;
	private javax.swing.JMenuItem newItem;
	private javax.swing.JMenuItem openItem;
	private javax.swing.JMenuItem saveItem;
	private javax.swing.JMenuItem saveAsItem;
	private javax.swing.JMenuItem exitMenuItem;
	private javax.swing.JCheckBoxMenuItem snapMenuItem;
	private javax.swing.JMenuItem settingsMenuItem;
	private javax.swing.JPanel jPanel1;
	private javax.swing.JPanel jResultsPanel;
	private PropertiesPanel propertiesPanel;
	private javax.swing.JList jResultsList;
	private javax.swing.JToolBar jToolBar;
	private javax.swing.JScrollPane jScrollPane2;
	private javax.swing.JScrollPane jScrollPane3;
	private javax.swing.JButton searchButton;
	private javax.swing.JPanel jPropertiesPanel;
	private javax.swing.JToolBar.Separator jSeparator1;
	private javax.swing.JTabbedPane jWrkFlwTabPane;
	private javax.swing.JPanel mainPanel;
	private javax.swing.JMenuBar menuBar;
	private javax.swing.JMenu fileMenu = new JMenu();
	private javax.swing.JMenu optionsMenu = new JMenu();
	private javax.swing.JMenu resultsMenu = new JMenu();
	private javax.swing.JProgressBar progressBar;
	private javax.swing.JLabel statusAnimationLabel;
	private javax.swing.JLabel statusMessageLabel;
	private javax.swing.JPanel statusPanel;
	// End of variables declaration//GEN-END:variables
	private final Timer messageTimer;
	private final Timer busyIconTimer;
	private final Icon idleIcon;
	private final Icon[] busyIcons = new Icon[15];
	private int busyIconIndex = 0;
	private JDialog aboutBox;
	private final Color selectedColor = new Color(19, 137, 203);
	private final EtchedBorder selectedBorder = new EtchedBorder(Color.white,
			selectedColor);
	private Border rolloverBorder;
	private Workflow workflow;
	private ResourceMap resourceMap = getResourceMap();
	private int tabsCount = 1;
	private TitledBorder propertiesBorder;
	private JFileChooser chooser = new JFileChooser(
			System.getProperty("user.dir") + File.separator + "data");

	public static enum EditingStatusType {
		DATASET, TOOL, MODEL, EDGE, DELETE, DRAG_PRESSED, DRAG_HOVER, SELECT
	}

	static EditingStatusType editingStatus = EditingStatusType.DATASET;

	public static EditingStatusType getEditingStatus() {
		return editingStatus;
	}

	public void setNodeProperties(WorkflowNode vertex) {

		propertiesPanel.clear();

		String title = "";
		if (vertex != null) {
			title = vertex.getName() + " properties";
			propertiesPanel.changeTable(vertex);
			propertiesBorder.setTitleJustification(TitledBorder.LEFT);
		} else {
			if (editingStatus == EditingStatusType.EDGE) {
				title = "edge";
			} else if (editingStatus == EditingStatusType.DELETE) {
				title = "delete";
			} else if (editingStatus == EditingStatusType.SELECT) {
				title = "select";
			} else {
				title = "-";
			}
			workflow.clearSelection();
			propertiesBorder.setTitleJustification(TitledBorder.RIGHT);
			propertiesPanel.clear();
		}
		propertiesBorder.setTitle(title);
		jPropertiesPanel.repaint();
	}

	public void setEdgeProperties(Edge edge) {
		workflow.clearSelection();
		propertiesPanel.clear();
		propertiesPanel.setEdgeType(edge);
		propertiesBorder.setTitleJustification(TitledBorder.LEFT);
		propertiesBorder.setTitle("Edge properties");

		jPropertiesPanel.repaint();
	}

	public void enableFileMenuItems() {
		newItem.setEnabled(true);
		openItem.setEnabled(true);
		saveItem.setEnabled(true);
	}

	public void setTabTitle(String s) {
		jWrkFlwTabPane.setTitleAt(0, s);
	}

	public void saveActionPerformed(ActionEvent event) {
		if (!workflow.isFileNameSet()) {
			saveAsActionPerformed(null);
		} else {
			workflow.saveToFile();
			setTabTitle(jWrkFlwTabPane.getTitleAt(0).substring(1));
			newItem.setEnabled(true);
			openItem.setEnabled(true);
			saveItem.setEnabled(false);
			saveAsItem.setEnabled(true);
		}
	}

	public void saveAsActionPerformed(ActionEvent event) {
		if (workflow.isFileNameSet()) {
			chooser.setSelectedFile(new File(workflow.getFileFullPath()));
		}

		int result = chooser.showSaveDialog(mainFrame);

		if (result == JFileChooser.APPROVE_OPTION) {
			String newFileName = chooser.getSelectedFile().getPath();

			if (!newFileName.endsWith(".xml")) {
				newFileName = newFileName.concat(".xml");
			}
			workflow.saveToFile(newFileName);
			String s = newFileName.substring(newFileName
					.lastIndexOf(File.separator) + 1);
			setTabTitle(s);
			mainFrame.setTitle(resourceMap.getString("Application.title")
					+ " - " + s);
			newItem.setEnabled(true);
			openItem.setEnabled(true);
			saveItem.setEnabled(false);
			saveAsItem.setEnabled(true);
		}
	}

	private void snapItemActionPerformed(ActionEvent evt) {
		workflow.setGridVisible(snapMenuItem.isSelected());
		workflow.setGridEnabled(snapMenuItem.isSelected());
		workflow.repaint();
	}

	private SettingsDialog dialog = new SettingsDialog();;

	private void settingsItemActionPerformed(ActionEvent evt) {
		// pop up dialog
		if (dialog.showDialog(mainFrame, "Settings")) {
			// if accepted, retrieve user input
		}
	}

	public void setStatus(String s) {
		statusMessageLabel.setText(s);
	}

	public void setSearchButtonEnabled(boolean enable) {
		searchButton.setEnabled(enable);
	}

	public static JFrame getMainFrame() {
		return mainFrame;
	}
}

class LeftAction extends AbstractAction {
	private static final long serialVersionUID = 1L;

	public LeftAction(String text, ImageIcon icon, String desc, Integer mnemonic) {
		super(text, icon);
		putValue(SHORT_DESCRIPTION, desc);
		putValue(MNEMONIC_KEY, mnemonic);
	}

	public LeftAction() {
		System.out.println("pippo");
	}

	@Override
	public synchronized PropertyChangeListener[] getPropertyChangeListeners() {
		System.out.println("pluto");
		return super.getPropertyChangeListeners();
	}

	public void actionPerformed(ActionEvent e) {
		System.out.println("Action for first button/menu item");
	}
}