/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.client.remote;

import java.net.InetAddress;
import java.net.UnknownHostException;

import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import org.globus.gsi.GlobusCredential;
import org.globus.gsi.GlobusCredentialException;
import org.globus.tools.proxy.GridProxyInit;

/**
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class GridProxy {

	public static void init() {

		if (!isValid()) {
			kgrid.client.gui.Utils
					.print("Proxy credentials expired: generating a new proxy...");
			final GridProxyInit proxy = new GridProxyInit();
			proxy.saveProxy(true);
			proxy.setCloseOnSuccess(true);
			proxy.pack();
			proxy.setLocation(400, 300);
			proxy.setModal(true);
			String sysLookAndFeel = UIManager.getSystemLookAndFeelClassName();
			try {
				UIManager.setLookAndFeel(sysLookAndFeel);
				SwingUtilities.updateComponentTreeUI(proxy);
			} catch (Exception e) {
				e.printStackTrace();
			}
			proxy.setVisible(true);
		} else {
			kgrid.client.gui.Utils.print("Proxy credentials are still valid");
		}
	}

	public static boolean isValid() {
		return timeLeft() > 0;

	}

	private static long timeLeft() {
		try {
			return getCredential().getTimeLeft();
		} catch (GlobusCredentialException e) {
			return 0;
		}
	}

	private static GlobusCredential getCredential()
			throws GlobusCredentialException {
		return GlobusCredential.getDefaultCredential();
	}
}
