/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.client.workflow;

import java.util.LinkedList;

import org.jdesktop.application.ResourceMap;
import org.jdom.Element;
import org.jdom.JDOMException;

/**
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class ModelNode extends WorkflowNode {

	private static final long serialVersionUID = 1L;
	private static final String NAME = "Model";
	private static final int TYPE = MODEL_TYPE;

	private LinkedList<Result> results = new LinkedList<Result>();

	public ModelNode(int id, int x, int y, ResourceMap resourceMap, Workflow w) {
		super(id, NAME, x, y, resourceMap, w);
		properties.put(LOCATION, "");
		properties.put(PRES, results);
	}

	public void addResult(String date, String fileName) {
		results.add(new Result(date, fileName));
		if (isEmpty()) {
			status = NOINFO;
		} else {
			status = LOCATION_SET;
		}
		updateIcon();
	}

	@Override
	public String getKDSURL() {
		return null;
	}

	@Override
	protected void setKDSURL(String s) {
		// do nothing
	}

	@Override
	public void clearSearchParameters() {
		// do nothing
	}

	public void setProperties(Object description) throws JDOMException {
		if (description != null) {
			properties.put(LOCATION, (String) description);
		}
		resetStatus();
	}

	@Override
	public String toString() {
		String hostname = (String) properties.get(LOCATION);
		if (!hostname.isEmpty()) {
			int i = hostname.indexOf(".");
			if (i > 0) {
				return hostname.substring(0, hostname.indexOf("."));
			}
			return hostname;
		}
		return null;
	}

	@Override
	public int getType() {
		return TYPE;
	}

	public boolean isEmpty() {
		return ((String) properties.get(LOCATION)).isEmpty()
				&& results.isEmpty();
	}

	@Override
	public void setProperty(int row, String property, String value) {
		properties.put(property, value);
		resetStatus();
	}

	@Override
	public GraphElement getElement() {
		GraphElement nodeE = new GraphElement("node", getIdName());
		nodeE.setAttribute("id", getIdName());
		Element typeE = new Element("data");
		typeE.setAttribute("key", "type");
		typeE.addContent("model");
		nodeE.addContent(typeE);
		if (!isEmpty()) {
			Element descriptionE = new Element("data");
			descriptionE.setAttribute("key", "location");
			descriptionE.setText((String) properties.get(LOCATION));
			nodeE.addContent(descriptionE);
			if (!results.isEmpty()) {
				for (Result res : results) {
					Element resE = new Element("data");
					resE.setAttribute("key", "result");
					resE.setAttribute("date", res.date);
					resE.setText(res.fileName);
					nodeE.addContent(resE);
				}
			}
		}
		addPosition(nodeE);
		return nodeE;
	}

	@Override
	public void resetStatus() {
		if (isEmpty()) {
			setStatus(NOINFO);
		} else {
			setStatus(LOCATION_SET);
		}
	}

	public class Result {
		String date;
		String fileName;

		public Result(String d, String f) {
			date = d;
			fileName = f;
		}

		public String getDate() {
			return date;
		}

		public String getFileName() {
			return fileName;
		}

	}
}
